/** 
* Voice engine used for speech synthesis.
* - `standard`: basic voice engine, less natural.
* - `neural`: realistic, neural-based synthesis.
* - `generative`: advanced AI voice with generative capacity.
*/
export type VoiceEngine = 'standard' | 'neural' | 'generative';

/** 
* Mode in which the audio is generated and received.
* - `block`: returns the entire audio at once.
* - `streaming`: audio is streamed progressively.
*/
export type VoiceMode = 'block' | 'streaming';

/**
* Gender of the voice used in synthesis.
* - `F`: female voice
* - `M`: male voice
*/
export type VoiceGender = 'F' | 'M';

/**
* Types of socket events emitted by the SDK.
* - `connected`: socket is connected.
* - `start`: The sockets are ready to receive data.
* - `end`: The sockets instance has ended.
* - `error`: an error occurred during processing.
* - `result`: result data is returned from the server.
*/
export type SocketEvents = 'connected' | 'start' | 'error' | 'result' | 'disconnect';

/**
* Structure representing an available voice for TTS.
*/
export type ResponseVoice = {
	/** Unique identifier for the voice. */
	uuid: string;

	/** Language identifier in ISO format (e.g., 'en-us'). */
	language: string;

	/** Gender of the voice. */
	gender: VoiceGender;

	/** Supported voice engines for this voice. */
	engines: Array<VoiceEngine>;

	/** Available output modes (block or streaming). */
	modes: Array<VoiceMode>;
}

/**
* Structure representing the result of a processed audio file.
*/
export type ResponseFile = {
	/** Transcribed text from the audio. */
	transcription: string;

	/** Language detected or provided for the source audio. */
	languageIdSource: string;

	/** Optional translated text. */
	translation?: string;

	/** Target language (if translation was requested). */
	languageIdTarget?: string;

	/** Optional binary audio response (e.g., TTS result). */
	file?: {
		type: string;
		data: ArrayBuffer;
	};
}

/**
 * Structure representing a supported language.
 */
export type ResponseLanguage = {
	/** Language identifier in ISO format (e.g., 'en-gb'). */
	id: string;

	/** Name of the language in English. */
	name: string;
}

/**
 * Configuration for the audio sent to the server.
 */
export interface SendAudio {
	/** How the audio is sent: in chunks or by a url. */
	broadcastVia: 'chunk' | 'url';

	/** Optional codec format used for the audio. Default linear16 */
	codec?: 'mp3' | 'linear16' | 'mulaw' | 'flac' | 'opus' | 'speex' | 'g729' | 'amr_nb' | 'amr_wb';

	/** Optional sample rate for the audio, between 8000 and 16000. Default 16000 */
	sampleRate?: number;

	/** Processing mode: phrase-based or continuous audio stream. Default phrase*/
	mode?: 'phrase' | 'continuous';

	/** If using broadcastVia = 'url', the location of the audio file. */
	url?: string;
}

/**
 * Configuration for the audio received from the server.
 */
export interface ReceiveAudio {
	/** How the audio is received: in chunks or by a URL. */
	obtainVia: 'chunk' | 'url';

	/** Optional gender of the synthesized voice. */
	gender?: VoiceGender;

	/** Optional codec format used for the received audio. Default linear16 */
	codec?: 'mp3' | 'linear16' | 'mulaw';

	/** Optional sample rate for the received audio between 8000 y 16000. Default 16000 */
	sampleRate?: number;

	/** Optional synthesis mode for TTS: block or streaming. Default block */
	mode?: VoiceMode;

	/** Optional voice engine used to generate the audio. Default neural */
	voiceEngine?: VoiceEngine;

	/** Optional UUID of the specific voice. */
	uuid?: string;
}

/**
 * Parameters required to initialize a socket connection for STT (Speech-to-Text).
 */
export interface STTSocketParams {
	/** Language spoken in the incoming audio */
	languageIdSource: string;

	sendAudio: SendAudio;
}

/**
 * Parameters required to initialize a socket connection for translation.
 */
export interface TranslatorSocketParams {
	/** Language spoken in the source text. */
	languageIdSource: string;

	/** Language into which will be translated. */
	languageIdTarget: string;
}

/**
 * Parameters required to initialize a socket connection for TTS (Text-to-Speech).
 */
export interface TTSSocketParams {
	/** Target language for audio synthesis. */
	languageIdTarget: string;

	receiveAudio: ReceiveAudio;
}

/**
 * Parameters required to initialize a socket connection for STS (Speech-to-Speech).
 */
export interface STSSocketParams {
	/** Language spoken in the source audio. */
	languageIdSource: string;

	/** Language into which the audio will be translated and synthesized. */
	languageIdTarget: string;

	sendAudio: SendAudio;
	receiveAudio: ReceiveAudio;
}



/**
* SpeaqrSDK is a lightweight client for interacting with the Speaqr API in real-time.
* It allows handling events, sending audio and text, and obtaining translations or speech synthesis.
*/
export declare class SpeaqrSDK {
	/**
	* Constructor to initialize an SDK instance.
	* 
	* @param params - Object containing the necessary parameters to configure the SDK.
	* @param params.apiKey - Your valid API key.
	* @param params.showLogs - (optional) Indicates whether to display logs in the console. Default is `false`.
	*/
	constructor(params: { 
		/** Your valid API key. */
		apiKey: string; 

		/** (optional) Indicates whether to display logs in the console. Default is `false`. */
		showLogs?: boolean; 
	});

	/**
	* Static method to asynchronously create an SDK instance with immediate API key validation.
	* 
	* @param params - Object containing the necessary parameters to configure the SDK.
	* @param params.apiKey - Your valid API key.
	* @param params.showLogs - (optional) Indicates whether to display logs in the console. Default is `false`.
	* @returns A promise that resolves with an instance of `SpeaqrSDK`.
	*/
	static create(params: { 
		/** Your valid API key. */
		apiKey: string;

		/** (optional) Indicates whether to display logs in the console. Default is `false`. */
		showLogs?: boolean;
	}): Promise<SpeaqrSDK>;
	
	/**
	* Releases all resources used by the instance and removes the methods
	* 
	* @returns A promise that is resolved when all resources have been released.
	*/
	destroy(): Promise<void>;

	/**
	* Gets the current API key.
	* 
	* @returns The current API key.
	*/
	get apiKey(): string;

	/**
	* Sets a new API key.
	* 
	* @param apiKey - The new API key to set.
	*/
	set apiKey(apiKey: string);	

	/**
	* Gets whether logs are enabled in the console.
	* 
	* @returns A boolean value indicating whether logs are enabled (`true`) or disabled (`false`).
	*/
	get logs(): boolean;

	/**
	* Enables or disables logs in the console.
	* 
	* @param show - A boolean value to enable (`true`) or disable (`false`) logs.
	*/
	set logs(show: boolean);
	
	/**
	* Returns whether sockets are open.
	* 
	* @returns A boolean value indicating whether sockets are open (`true`) or closed (`false`).
	 */
	get socketOpened(): boolean;

	/**
	* Retrieves the list of languages supported by the SDK.
	* 
	* @returns A promise that resolves with an array of objects like {id: 'en-gb', name: 'English'}.
	* The `name` property contains the language name in its own language.
	*/
	languages(): Promise<ResponseLanguage[]>;

	/**
	* Returns the voices available for a given configuration between “language/voiceEngine/voiceGender/ttsEngine”.
	* 
	* @param params - Object with filters to list voices.
	* @returns A promise that resolves to an array of objects with the available voices like:
	* {
	* 	"uuid": "4494c710-61c8-4f22-968f-20f021e45052",
	* 	"language": "es-es",
	* 	"gender": "M",
	* 	"engines": [
	* 		"neural",
	* 		"generative"
	* 	],
	* 	"modes": [
	* 		"block",
	* 		"streaming"
	* 	]
	* }
	*/
	voices(
		/** The language in ISO3 format, e.g., `en-gb`. */
		language: string,

		/** (optional) How to generate audio 'block', or 'streaming'. */
		mode?: VoiceMode,

		/** (optional) Voice gender: 'F' (female), or 'M' (male). */
		voiceGender?: VoiceGender,

		/** (optional) Voice engine for synthesis: 'standard', 'neural', 'generative'. */
		voiceEngine?: VoiceEngine[]
	): Promise<ResponseVoice[]>;

	/**
	* Processes an audio file and performs the specified operation (sts, stt, or stt+translation).
	* 
	* @param params - Object containing the necessary parameters to process the file.
	* @returns A promise that resolves with an object whose properties depend on `mode`.
	{
		"transcription": "hello this is a test audio recording",
		"languageIdSource": "es-es",
		"languageIdTarget": "en-us",
		"translation": "Hi, this is an audio recording test.",
		"file": {
			"type": "Buffer",
			"data": ArrayBuffer
		}
	}
	*/
	file(params: { 
		/** (optional) Processing mode: 'sts', 'stt', or 'stt+translation'. Default sts.*/
		mode?: 'sts' | 'stt' | 'stt+translation';

		/** (optional) The source language in ISO3 format, e.g., `en-gb`. If not passed, an attempt will be made to detect the audio language. */
		languageIdSource?: string;

		/** (required for mode sts y stt+translation) The target language in ISO3 format, e.g., `en-gb`. */
		languageIdTarget?: string;

		/** (optional) Voice gender: 'F' (female), or 'M' (male). */
		voiceGender?: VoiceGender;

		/** Audio file to process, in Blob format. */
		file: Blob;
	}): Promise<ResponseFile>;


	/**
	* Initializes a socket instance for real-time communication with the API.
	* 
	* @param namespace - The namespace to instantiate.
	* @param params - Object containing the necessary parameters to configure the socket instance.
	* @returns A promise that resolves when the instance is ready.
	*/
	socketsInstance(namespace: 'stt', params: STTSocketParams): Promise<void>;
	socketsInstance(namespace: 'translator', params: TranslatorSocketParams): Promise<void>;
	socketsInstance(namespace: 'tts', params: TTSSocketParams): Promise<void>;
	socketsInstance(namespace: 'sts', params: STSSocketParams): Promise<void>;

	/**
	* Closes the socket connection previously opened.
	* 
	* @returns A promise that resolves when the connection is successfully closed.
	*/
	socketsClose(): Promise<void>;
	
	/**
	* Subscribes a listener to a specific socket event.
	* 
	* @param eventType - The type of event to subscribe to.
	* @param listener - The function to be called when the event occurs. Receives an object that will depend on the event and the connection mode. See the documentation http://speaqr.ai/en-us/technical/sdks/javascript
	*/
	socketsAddListener(eventType: SocketEvents, listener: (data: any) => void): void;

	/**
	* Unsubscribes a listener from a specific socket event.
	* 
	* @param eventType - The type of event to unsubscribe from.
	* @param listener - The function previously associated with the event.
	*/
	socketsRemoveListener(eventType: SocketEvents, listener: (data: any) => void): void;

	
	/**
	* Sends a text through the socket connection to be processed.
	* 
	* @param text - The text to be sent.
	* @returns A promise that resolves with the processing result.
	*/
	socketsSendText(text: string): Promise<boolean>;
	
	/**
	* Sends an audio fragment in the form of an ArrayBuffer through the socket connection to be processed.
	* 
	* @param data - The audio fragment to be sent to the server.
	* @returns A promise that resolves with the processing result.
	*/
	socketsSendStream(data: ArrayBuffer | Blob):  Promise<boolean>;
}
