/*! 
SDK for accessing Speaqr's public API from Javascript / Node.js
(c) 2024 Speaqr Technologies S.L.
version 2.0.0.

Commercial website: https://speaqr.ai
Technical documentation: https://speaqr.ai/en-us/technical
*/
(function (global, factory) {
	if (typeof define === 'function' && define.amd) {
	  // AMD (RequireJS)
	  define([], factory);
	} else if (typeof exports === 'object' && typeof module !== 'undefined') {
	  // CommonJS / Node
	  module.exports = { SpeaqrSDK: factory() };
	} else {
	  // Browser global
	  global.SpeaqrSDK = factory();
	}
  })(this, function () {

	//========================================================
	//
	//
	//
	//				speaqrSDK class
	//
	//
	//
	//
	//========================================================
	class SpeaqrSDK {
		#apiKey = null;
		#showLogs = false;
		static #publicApi = typeof ENV != 'undefined' && typeof ENV.URL_API_PUBLIC != 'undefined' ? ENV.URL_API_PUBLIC : 'https://public.speaqr.ai';
		#Languages = null;	//Languages list
		#Sockets = null;	//socketIO instance
		#SocketsOpen = false;
		#SocketsOpenResponse = null;
		#socketListeners = null;
		static #SOCKETIO_SRC = 'https://cdn.socket.io/4.8.1/socket.io.min.js';
		static #SOCKETIO_INTEGRITY = 'sha384-mkQ3/7FUtcGyoppY6bz/PORYoGqOl7/aSUMn2ymDOJcapfS6PHqxhRTMh1RR0Q6+';
		static #SOCKETS_EVENTS = ['connected', 'start', 'error', 'result', 'disconnect'];

		//============================================
		// Constructor
		//============================================
		constructor({apiKey, showLogs=false}) {
			if (typeof apiKey!='string'){
				throw new Error('An apiKey needs to be provided to instantiate the class.');
			}

			this.#apiKey = apiKey;
			this.#showLogs = showLogs;
		}

		//============================================
		// async constructor with apiKey check
		//============================================
		static async create({apiKey, showLogs=false}){
			if (typeof apiKey!='string'){
				throw new Error('An apiKey needs to be provided to instantiate the class.');
			}

			return new Promise((resolve, reject) => {
				fetch(`${SpeaqrSDK.#publicApi}/api/languages`, {
					headers: { 'x-api-key': `${apiKey}` }
				})
				.then((response) => {
					response.json()
					.then( (json) => {
						if (this.#checkError(json)){
							reject(json);
						}else{
							const instance = new SpeaqrSDK({apiKey, showLogs});
							instance.#Languages = json;
							resolve(instance);
						};
					})
					.catch( (err) => {
						reject(err);
					});
				})
				.catch((err) => {
					reject(err);
				});
			});
		}

		//============================================
		// Free up class resources
		//============================================
		destroy(){
			return new Promise(async (resolve, reject) => {
				try {
					await this.socketsClose();
				} catch (error) {
				}
	
				this.#Languages = null;
				this.#apiKey = null;
	
				Object.getOwnPropertyNames(SpeaqrSDK.prototype).forEach((method) => {
					if (method !== 'constructor' && typeof this[method] === 'function') {
						this[method] = null;
					}
				});
			
				this.#paintLog('SDK instance successfully destroyed.', 'log');

				resolve();
			})
		}

		//============================================
		// Properties
		//============================================

		//--------------------------------------------
		// The apiKey 
		//--------------------------------------------
		get apiKey(){
			return this.#apiKey;
		}
		set apiKey(apiKey){
			this.#apiKey = apiKey;
		}

		//--------------------------------------------
		// Whether or not to display console logs
		//--------------------------------------------
		get logs(){
			return this.#showLogs;
		}
		set logs(show){
			if (typeof show!='boolean'){
				throw new Error('The value of the showLogs property must be boolean.');
			}

			this.#showLogs = show;
		}

		//--------------------------------------------
		// Returns whether sockets are 
		// open (true) or closed (false)
		//--------------------------------------------
		get socketOpened(){
			return this.#SocketsOpen;
		}

		//============================================
		// Returns the supported languages
		//============================================
		languages() {
			return new Promise((resolve, reject) => {
				if (this.#Languages){
					resolve(this.#Languages);
					return;
				}

				fetch(`${SpeaqrSDK.#publicApi}/api/languages`, {
					headers: { 'x-api-key': `${this.#apiKey}` }
				})
				.then((response) => {
					response.json()
					.then( (json) => {
						if (this.#checkErrorAndPaint(json)){
							reject(json);
						}else{
							this.#Languages = json;
							resolve(json);
						};
					})
					.catch( (err) => {
						this.#paintLog(err);
						reject(err);
					});
				})
				.catch((err) => {
					this.#paintLog(err);
					reject(err);
				});
			});
		}

		//=====================================================================
		// Returns the voices available for a given 
		// configuration between “language/voiceEngine/voiceGender/ttsEngine”
		//
		//	language:"es-es",
		//	voiceGender:"M",
		//	mode:"block",
		//	voiceEngine:["standard"]
		//=====================================================================
		voices(language, mode, voiceGender, voiceEngine) {
			return new Promise((resolve, reject) => {
				const params = new URLSearchParams();

				if (language) params.append('language', language);
				if (mode) params.append('mode', mode);
				if (voiceGender) params.append('voiceGender', voiceGender);
				if (voiceEngine){
					if (!Array.isArray(voiceEngine)) voiceEngine = [voiceEngine];
					voiceEngine.forEach(engine => {
						params.append('voiceEngine', engine);
					});
				} 
		
				// Construir URL completa con query
				const url = `${SpeaqrSDK.#publicApi}/api/voices?${params.toString()}`;

				fetch(url, {
					headers: {
						'x-api-key': `${this.#apiKey}`,
						'Content-Type': 'application/json',
					},
					method: 'GET',
				})
				.then(
					(response) => {
						response.json()
						.then( (json) => {
							if (this.#checkErrorAndPaint(json)){
								reject(json);
							}else{
								resolve(json);
							}
						});
					}
				)
				.catch( (err) => {
					this.#paintLog(err);
					reject(err);
				});
			});
		}

		//==================================================
		// Processes an audio wave containing an mp3 or wav
		//	params = {
		// 		mode: 'sts',
		// 		languageIdSource: 'en-us',
		// 		languageIdTarget: 'en-us',
		//		voiceGender: 'M'
		// 		file: (ArrayBuffer)
		//	}
		//==================================================
		file(params) {
			const formData = new FormData();
			if (params.mode) formData.append('mode', params.mode);
			if (params.languageIdSource) formData.append('languageIdTarget', params.languageIdSource);
			if (params.languageIdTarget) formData.append('languageIdTarget', params.languageIdTarget);
			if (params.voiceGender) formData.append('voiceGender', params.voiceGender);
			formData.append('file', params.file);

			return new Promise((resolve, reject) => {
				fetch(`${SpeaqrSDK.#publicApi}/api/batch/file`, {
					headers: {
						'x-api-key': `${this.#apiKey}`
					},
					method: 'POST',
					body: formData,
				})
				.then(
					(response) => {
						response.json()
						.then( (json) => {
							if (this.#checkErrorAndPaint(json)){
								reject(json);
							}else{
								if (json.file && json.file.data){
									const int8 = new Uint8Array(json.file.data);
									json.file.data = int8.buffer;
								};
			
								resolve(json);
							}
						});
					}
				)
				.catch( (err) => {
					this.#paintLog(err);
					reject(err);
				});
			});
		}


		//==========================================================================
		// Instantiating sockets
		//==========================================================================
		socketsInstance(namespace, params) {
			if (this.#Sockets) throw new Error('A connection to the sockets is already established.');

			if (params.sendAudio) params.sendAudio = JSON.stringify(params.sendAudio);
			if (params.receiveAudio) params.receiveAudio = JSON.stringify(params.receiveAudio);

			this.#paintLog('Connecting to sockets. Params: ' + JSON.stringify(params), 'debug');

			this.#socketListeners = [];

			return new Promise((resolve, reject) => {
				this.#loadSocketIO()
				.then( () => {
					
					this.#Sockets = io(SpeaqrSDK.#publicApi + '/' + namespace, {
						transports: ['websocket'],
						upgrade: false,
						auth: {token: this.#apiKey},
						autoConnect: false,
						reconnection: false,
						forceNew: true,
						query: params
					});
		
					resolve();
				})
				.catch( (err) => {
					this.#paintLog(err);
					reject(err);
				});
			});
		}

		//==========================================================================
		// Connection to sockets
		//==========================================================================
		socketsOpen() {
			if (!this.#Sockets) throw new Error('Sockets are not instantiated.');

			return new Promise((resolve, reject) => {
				if (this.#SocketsOpen){
					resolve(this.#SocketsOpenResponse);
					return;
				};

				this.#Sockets.on('connect', this.#s_on_connect.bind(this));
				this.#Sockets.on('disconnect', this.#s_on_disconnect.bind(this));
				this.#Sockets.on('error', this.#s_on_error.bind(this));
				
				this.#Sockets.on('connected', this.#s_on_connected.bind(this));
				this.#Sockets.on('start', (data) => {
					this.#SocketsOpenResponse = data;
					this.#paintLog(`Sockets ready to receive data. ${data}`, 'debug');
					resolve(data);
				});

				this.#Sockets.on('end', this.#s_on_end.bind(this));


				this.#Sockets.on('connect_error', (err) => {
					this.#paintLog(err);
					reject(err);
				});

				this.#Sockets.connect(
					(err) => {
						this.#paintLog(err);
					}
				);
			});
		}

		//============================================
		// Disconnecting the sockets
		//============================================
		socketsClose() {
			if (!this.#Sockets) throw new Error('Sockets are not instantiated.');

			return new Promise((resolve) => {
				if (this.#SocketsOpen){
					this.#Sockets.on('disconnect', resolve);
					this.#Sockets.disconnect();
				}else{
					this.#Sockets.disconnect();
					resolve();
				};
			});
		}

		//============================================
		// Subscribe to a sockets event
		//============================================
		socketsAddListener(eventType, listener){
			if (!this.#Sockets) throw new Error('Sockets are not instantiated.');

			if (!SpeaqrSDK.#SOCKETS_EVENTS.indexOf(eventType)==-1) throw new Error(`The event ${eventType} does not exist.`);
			if (this.#s_search_listener(eventType, listener)!==-1) throw new Error(`The event ${eventType} already has that listener assigned.`);

			this.#Sockets.on(eventType, listener);
			this.#socketListeners.push({eventType, listener});
		}

		//============================================
		// Unsubscribes from a socket event
		//============================================
		socketsRemoveListener(eventType, listener){
			if (!this.#Sockets) throw new Error('Sockets are not instantiated.');

			if (!SpeaqrSDK.#SOCKETS_EVENTS.indexOf(eventType)==-1) throw new Error(`The event ${eventType} does not exist.`);
			const idx = this.#s_search_listener(eventType, listener)
			if (idx==-1) throw new Error(`The event ${eventType} does not have that listener assigned.`);

			this.#Sockets.off(eventType, listener);
			this.#socketListeners.slice(idx, 1);
		}

		//============================================
		// Sending text to sockets
		//============================================
		socketsSendText(text) {
			if (!this.#Sockets) throw new Error('Sockets are not instantiated.');
			if (!this.#SocketsOpen) throw new Error('Sockets are not opened.');

			return new Promise((resolve, reject) => {
				this.#Sockets.emit('text', text, (response) => {
					this.#paintLog('socketsSendText response', 'log');
					if (response.error) {
						this.#paintLog(response);
						reject(response);
					} else {
						this.#paintLog(response, 'log');
						resolve(response);
					}
				});
			});
		}

		//============================================
		// Sends a buffer array with an audio chunk
		//============================================
		socketsSendStream(arrayBuffer) {
			if (!this.#Sockets) throw new Error('Sockets are not instantiated.');
			if (!this.#SocketsOpen) throw new Error('Sockets are not opened.');

			return new Promise((resolve, reject) => {
				this.#Sockets.emit('write_stream', arrayBuffer, (response) => {
					this.#paintLog('socketsSendStream response', 'log');
					if (response.error) {
						this.#paintLog(response);
						reject(response);
					} else {
						this.#paintLog(response, 'log');
						resolve(response);
					}
				});
			});
		}	

		//============================================
		// Socket events to control
		//============================================
		#s_on_connect(){
			this.#SocketsOpen = true;
			this.#paintLog('Connected to Speaqr socket server', 'debug');
		}
		#s_on_disconnect(reason){
			this.#paintLog(`Disconnected from Speaqr socket server. Reason:${reason}`, 'debug');
			this.#Sockets = null;
			this.#socketListeners = null;
			this.#SocketsOpen = false;
			this.#SocketsOpenResponse = null;
		}
		#s_on_error(err){
			this.#paintLog(err);
		}

		#s_on_connected(status){
			this.#paintLog(`on_connected`, 'debug', status);
		}
		#s_on_end(any){
			this.#paintLog(`on_end`, 'debug', any);
		}


		#s_search_listener(eventType, listener){
			const index = this.#socketListeners.findIndex(
				(obj) => obj.eventType === eventType && obj.listener === listener
			);
			return index;
		}

		//============================================
		// Load SocketIO dynamically
		//============================================
		#loadSocketIO() {
			return new Promise((resolve, reject) => {
				//We check if we are in a browser
				if (
					typeof document !== 'undefined' &&
					typeof document.nodeType !== 'undefined' &&
					typeof window !== 'undefined' &&
					window.document === document &&
					document.nodeType === 9
				){
					//Browser
					this.#loadSocketIOBrowser()
					.then( resolve )
					.catch( reject );
				}else{
					//We check if we are in Node
					if (
						typeof process !== 'undefined' &&
						typeof process.versions !== 'undefined' &&
						typeof process.versions.node !== 'undefined'
					  ){
						this.#loadSocketIONode()
						.then( resolve )
						.catch( reject );
					  }else{
						reject('Socket IO cannot be loaded dynamically. You must instantiate it in your project.');
					  }
				}
			});
		}

		//--------------------------------------------
		// Load SocketIO for browser
		//--------------------------------------------
		#loadSocketIOBrowser() {
			return new Promise((resolve, reject) => {
				if (typeof io!='undefined'){	//If it is already present we do not load it
					resolve();
					return;
				}

				const script = document.createElement('script');
				script.src = SpeaqrSDK.#SOCKETIO_SRC;
				script.integrity = SpeaqrSDK.#SOCKETIO_INTEGRITY;
				script.crossOrigin = 'anonymous';

				// When the script is loaded successfully
				script.onload = () => resolve();
				
				// En caso de error durante la carga
				script.onerror = () => reject(new Error(`Failed to load script ${script.src}`));

				// We add the script to the document
				document.head.appendChild(script);
			});
		}

		//--------------------------------------------
		// Load SocketIO for Node
		//--------------------------------------------
		#loadSocketIONode() {
			return new Promise((resolve, reject) => {
				if (typeof globalThis.io !== 'undefined') {
					resolve();
					return;
				}

				try {
					const { io } = require('socket.io-client');
					globalThis.io = io;
					resolve();
				} catch (error) {
					reject(new Error(`Failed to load socket.io-client on Node. Please install with "npm install socket.io-client". Error: ${error}`));
				}
			});
		}		

		//============================================
		// Check if an api response is an error
		//============================================
		static #checkError(json){
			return (typeof json=='object' && typeof json.error!='undefined')
		}

		//============================================
		// Check if an api response is 
		// an error and paint it
		//============================================
		#checkErrorAndPaint(json){
			if (SpeaqrSDK.#checkError(json)){
				this.#paintLog(json);
				return true;			
			}

			return false;
		}

		//============================================
		// Display logs on console
		//============================================
		#paintLog(log, type='error', object = '') {
			if (!this.#showLogs) return;

			console[type]('Speaqr SDK', log, object);
		}
	}


  // Devuelve la clase
  return SpeaqrSDK;
});